@@ -22,10 +22,13 @@
 // Clippy
 #![warn(clippy::pedantic)]
 
+#[macro_use]
+mod util;
 pub mod read_only;
 
 use std::borrow::{Borrow, BorrowMut};
 use std::cmp::Ordering;
+use std::fmt::Display;
 use std::ops::{Deref, DerefMut};
 
 /// A smart pointer that either owns or mutably borrows.
@@ -106,13 +109,7 @@ impl<'b, T> Borrowned<'b, T> {
     }
 }
 
-impl<'b, T> Deref for Borrowned<'b, T> {
-    type Target = T;
-
-    fn deref(&self) -> &Self::Target {
-        self.inner_ref()
-    }
-}
+shared_impls!();
 
 impl<'b, T> DerefMut for Borrowned<'b, T> {
     fn deref_mut(&mut self) -> &mut Self::Target {
@@ -120,79 +117,18 @@ impl<'b, T> DerefMut for Borrowned<'b, T> {
     }
 }
 
-impl<'b, T> Borrow<T> for Borrowned<'b, T> {
-    fn borrow(&self) -> &T {
-        self.inner_ref()
-    }
-}
-
 impl<'b, T> BorrowMut<T> for Borrowned<'b, T> {
     fn borrow_mut(&mut self) -> &mut T {
         self.inner_mut()
     }
 }
 
-impl<'b, T> AsRef<T> for Borrowned<'b, T> {
-    fn as_ref(&self) -> &T {
-        self.inner_ref()
-    }
-}
-
 impl<'b, T> AsMut<T> for Borrowned<'b, T> {
     fn as_mut(&mut self) -> &mut T {
         self.inner_mut()
     }
 }
 
-impl<'b, T: Clone> Clone for Borrowned<'b, T> {
-    fn clone(&self) -> Self {
-        match self {
-            Borrowned::Owned(owned) => Borrowned::Owned(owned.clone()),
-            Borrowned::Borrowed(borrowed) => {
-                let borrowed = *borrowed as &T;
-                Borrowned::Owned(borrowed.clone())
-            }
-        }
-    }
-}
-
-impl<'b, T: PartialEq> PartialEq for Borrowned<'b, T> {
-    fn eq(&self, other: &Self) -> bool {
-        let b_self = self.inner_ref();
-        let b_other = other.inner_ref();
-
-        b_self.eq(b_other)
-    }
-}
-
-impl<'b, T: PartialOrd> PartialOrd for Borrowned<'b, T> {
-    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        let b_self = self.inner_ref();
-        let b_other = other.inner_ref();
-
-        b_self.partial_cmp(b_other)
-    }
-}
-
-impl<'b, T: Default> Default for Borrowned<'b, T> {
-    fn default() -> Self {
-        Self::Owned(T::default())
-    }
-}
-
-impl<'i, 'b, T> IntoIterator for &'i Borrowned<'b, T>
-where
-    &'i T: IntoIterator,
-{
-    type Item = <&'i T as IntoIterator>::Item;
-
-    type IntoIter = <&'i T as IntoIterator>::IntoIter;
-
-    fn into_iter(self) -> Self::IntoIter {
-        self.inner_ref().into_iter()
-    }
-}
-
 impl<'i, 'b, T> IntoIterator for &'i mut Borrowned<'b, T>
 where
     &'i mut T: IntoIterator,
